//#include <pspsdk.h>
#include <pspkernel.h>
#include <pspctrl.h>
#include <psppower.h>
#include <stdio.h>
#include <string.h>
//#include <stdlib.h>
#include <stdarg.h>

//Has the prx's functions
#include "id_driver.h"

PSP_MODULE_INFO("IdStorageChange", 0, 1, 0);
PSP_MAIN_THREAD_ATTR(PSP_THREAD_ATTR_USER);

#define printf pspDebugScreenPrintf

void ErrorExit(int milisecs, char *fmt, ...)
{
	va_list list;
	char msg[256];	

	va_start(list, fmt);
	vsprintf(msg, fmt, list);
	va_end(list);

	printf(msg);

	sceKernelDelayThread(milisecs*1000);
	sceKernelExitGame();
}

void ExitWhen(char *fmt, ...)
{
	SceCtrlData pad;
	va_list list;
	char msg[256];	

	va_start(list, fmt);
	vsprintf(msg, fmt, list);
	va_end(list);

	printf(msg);

	while (1)
	{
		sceCtrlReadBufferPositive(&pad, 1);
		if (pad.Buttons >= 0x001000 && pad.Buttons <= 0x008000 )
		{
			sceKernelExitGame();
		}

		sceKernelDelayThread(50000);
	}
	sceKernelExitGame();
}

int LoadStartModule(char *module)
{
	SceUID mod = sceKernelLoadModule(module, 0, NULL);

	if (mod < 0)
		return mod;

	return sceKernelStartModule(mod, strlen(module)+1, module, NULL, NULL);
}

void Agreement()
{
	SceCtrlData pad;
	
	pspDebugScreenClear();
	printf("You are about to change the idstorage of your PSP.\n");
	printf("Press X to start. By doing it, you accept the risk and ALL the responsability of what happens.\n");
	printf("If you don't agree press R button.\n");

	while (1)
	{
		sceCtrlReadBufferPositive(&pad, 1);

		if (pad.Buttons & PSP_CTRL_CROSS)
		{
			return;			
		}

		else if (pad.Buttons & PSP_CTRL_RTRIGGER)
		{
			sceKernelExitGame();
		}

		sceKernelDelayThread(10000);
	}
}

void Detect(char *msg)
{
	SceCtrlData pad;

	printf("This Program Has Detected That\n   ");
	printf(msg);

	printf("And Will Modify Your IdStorage Based on this\n");
	printf("\n****IF THIS IS NOT CORRECT EXIT NOW****\n\n");
	printf("Exit = RIGHT TRIGGER\nContinue = SQUARE\n");

	while (1)
	{
		sceCtrlReadBufferPositive(&pad, 1);

		if (pad.Buttons & PSP_CTRL_SQUARE)
		{
			return;			
		}

		else if (pad.Buttons & PSP_CTRL_RTRIGGER)
		{
			sceKernelExitGame();
		}
		sceKernelDelayThread(10000);
	}
}

void Attempt (char sym)
{
	int ret = -9;
	
	printf("Running Fix to Repair a PSP that Has ");

	if (sym == 'U')
	{
		printf("Run the original Downgrader and the USB Fix\n"); 
		ret = pspTA082FixUSBFix();
	}
	else if (sym == 'I')
	{	
		printf("Not run any Idstorage fixers / downgraders\n"); 
		ret = pspTA082Enable15Ipl();
	}
	else if (sym == 'R')
	{
		printf("Only Run the original DAX downgrader\n");
		ret = pspTA082FixIdStorage();
	}
	else if (sym == 'N')
	{
		printf("Run ONLY the modified Noobz downgrader\n");
		ret = pspTA082FixNoobzDG();
	}
	else if (sym == 'V')
	{
		printf("Ran the old version of this dg\n");
		ret = pspTA082FixOfFix();
	}
	else if (sym == 'S')
	{
		printf("Ran Stapol's RUK\n");
		ret = pspTA082FixOfRUK();
	}
	else
	{
		pspDebugScreenClear();
		ErrorExit(6000, "INTERNAL ERROR IN FUNCTION ATTEMPT");
	}


	if (ret < 0)
	{
		ErrorExit(6000, "Error Preforming Task!!");
	}

	ErrorExit(6000, "\n\n    *******Task Completed******");
	return;
	
}

int main() 
{
	pspDebugScreenInit();


	if (scePowerGetBatteryLifePercent() < 75)
	{
		ErrorExit(6000, "Battery has to be at least at 75%%.\n");
	}
	
	int res = LoadStartModule("id_driver.prx");
	
	if (res < 0)
	{
		ErrorExit(6000, "Error loading/starting id_driver.prx.\n");
	}
	
	if (!pspTA082IsTA082())
	{
		ErrorExit(6000, "Your PSP is not A TA082. Use of this Fixer Should not be Required\n");
	}
	int is42a6bup    = is42backup6();
	int key4isRight  = is4right();
	int key6isRight  = is6right();
	int key5isGood   = is5good();
	int isDGall		 = isFinalDGed();
	int key43isRight = is43right();
	int key47isRight = is47right();
	int key41isRight = is41right();
	int key45isRight = is45right();
	int canRun15	 = pspTA082CanRun15Ipl();

//	Test Functions
	
	printf("42 is a 6 b-up: %d \n", is42a6bup);
	printf("is6right      : %d \n", key6isRight);
	printf("is4right      : %d \n", key4isRight);
	printf("key 5 good?   : %d \n", key5isGood);	
	printf("isFinalDGed   : %d \n", isDGall);
	printf("is43right     : %d \n", key43isRight);
	printf("is47right     : %d \n", key47isRight);
	printf("is41right     : %d \n", key41isRight);
	printf("is45right     : %d \n", key45isRight);
	printf("Can Run 1.5   : %d \n", canRun15);

	if (isDGall)
	{
		ErrorExit(6000, "Your PSP is already Repaired\n");
	}
	

	if (key43isRight && key47isRight && key41isRight && (!key45isRight) && canRun15 && !key6isRight && !key5isGood)
	{
		Detect("Your PSP Has Run The *** USB FIX ***\n");
		Agreement();
		Attempt('U');
	}
	else if (key43isRight && key47isRight && key41isRight && key45isRight && canRun15 && !key4isRight && !key5isGood && !key6isRight &&  !is42a6bup)
	{
		Detect("Your PSP Has Run The *** NOOBZ FIX ***\n");
		Agreement();
		Attempt('N');
	}
	else if ((!key43isRight) && (!key47isRight) && (!key41isRight) && (!key45isRight) && canRun15 && !key4isRight && !key5isGood && !key6isRight && ! is42a6bup)
	{
		Detect("Your PSP Has Run The *** ORINGINAL DG ONLY ***\n");
		Agreement();
		Attempt('R');
	}
	else if (key43isRight && key47isRight && key41isRight && key45isRight && (!canRun15) && key4isRight && key6isRight && ! is42a6bup)
	{
		Detect("Your PSP Has NOT RUN ANY ID STORAGE CHANGING PROGRAMS\n");
		Agreement();
		Attempt('I');
	}
	else if (key4isRight && key43isRight && key47isRight && key41isRight && key45isRight && canRun15 && !key6isRight && !key5isGood &&  is42a6bup)
	{
		Detect("PSP has run the last version of this program\n");
		Agreement();
		Attempt('V');
	}
	else if (key4isRight && key43isRight && key47isRight && key41isRight && key45isRight && canRun15 && !key6isRight && !key5isGood && !is42a6bup)
	{
		Detect("PSP has run Stapol's RUK\n");
		Agreement();
		Attempt('S');
	}
	else
	{
		ExitWhen("Your IdStorage is Too messed up For me to screw with\nExiting For my Sanity and Yours\n");
	}

	return 0;
}

