#include <pspsdk.h>
#include <pspkernel.h>
#include <pspidstorage.h>
#include <string.h>

#include "idkey_43.h"
#include "idkey_4_82.h"
#include "idkey_5_82.h"
#include "idkey_6_82.h"

PSP_MODULE_INFO("id_driver", 0x1006, 1, 0);
PSP_MAIN_THREAD_ATTR(0);

u8 leaf[512];
u8 leaf2[512];

int pspTA082FixOfRUK()
{
	int k1 = pspSdkSetK1(0);
	
	if (sceIdStorageIsReadOnly())
		return -1;
	
	sceIdStorageReadLeaf(0x6,leaf);
	leaf[1]=0x68;
	sceIdStorageWriteLeaf(0x5,leaf);

	sceIdStorageWriteLeaf(0x6, idkey_6_82);

	pspSdkSetK1(k1);
	return 0;
}

int pspTA082FixOfFix()
{
	int k1 = pspSdkSetK1(0);
	
	if (sceIdStorageIsReadOnly())
		return -1;
	
	sceIdStorageReadLeaf(0x6,leaf);
	leaf[1]=0x68;
	sceIdStorageWriteLeaf(0x5,leaf);

	sceIdStorageReadLeaf(0x42,leaf);
	sceIdStorageWriteLeaf(0x6,leaf);


	pspSdkSetK1(k1);
	return 0;
}

	
int pspTA082Enable15Ipl()
{
	int k1 = pspSdkSetK1(0);	
	
	if (sceIdStorageIsReadOnly())
		return -1;

	//Change 0x5 so it appears corrupt 
	sceIdStorageReadLeaf(0x5,leaf);
	leaf[1]=0x68; //From 6B
	sceIdStorageWriteLeaf(0x5,leaf);

	pspSdkSetK1(k1);
	return 0;
}

int pspTA082FixUSBFix()
{
	int k1 = pspSdkSetK1(0);	
	
	if (sceIdStorageIsReadOnly())
		return -1;
	
	// key 0x0004 fix
	sceIdStorageReadLeaf(0x0005,leaf);
	sceIdStorageWriteLeaf(0x0004,leaf);

	// key 5 corrupt restore
	sceIdStorageReadLeaf(0x6,leaf);
	leaf[1]=0x68;
	sceIdStorageWriteLeaf(0x5,leaf);

	// Key 6 restore (it is lost in the idstorage fix)
	sceIdStorageWriteLeaf(0x6, idkey_6_82);

	// key 0x45 fix
    sceIdStorageReadLeaf(0x0046,leaf);
	sceIdStorageWriteLeaf(0x0045,leaf);

	pspSdkSetK1(k1);
	return 0;
}

int pspTA082FixIdStorage()
{
	int k1 = pspSdkSetK1(0);	
	
	if (sceIdStorageIsReadOnly())
		return -1;
	
	//Key 0x0047 fix

	memset(leaf,0,512);
	leaf[0] = 0x09;

	sceIdStorageWriteLeaf(0x0047, leaf);
	

	// key 0x0004 fix
	sceIdStorageReadLeaf(0x0005,leaf);
	sceIdStorageWriteLeaf(0x0004,leaf);

	// key 5 + 6
	sceIdStorageReadLeaf(0x6,leaf);
	leaf[1]=0x68;
	sceIdStorageWriteLeaf(0x5,leaf);

	sceIdStorageReadLeaf(0x41, leaf);
	sceIdStorageWriteLeaf(0x6,leaf);
	

	/**  Key 0x0043 Fix  ***************/


	sceIdStorageWriteLeaf(0x0043, idkey_43);

    /***********************************/

	// key 0x0041, 0x0042 swap (a back up of 0x6 is now in 0x42) 
	sceIdStorageReadLeaf(0x0041,leaf);
	sceIdStorageReadLeaf(0x0042,leaf2);
	sceIdStorageWriteLeaf(0x0042,leaf);
	sceIdStorageWriteLeaf(0x0041,leaf2);

	// key 0x45 fix
    sceIdStorageReadLeaf(0x0046,leaf);
	sceIdStorageWriteLeaf(0x0045,leaf);

	pspSdkSetK1(k1);
	return 0;
}

int pspTA082FixNoobzDG()
{
	int k1 = pspSdkSetK1(0);

	if (sceIdStorageIsReadOnly())
		return -1;

	
	sceIdStorageReadLeaf(0x5,leaf);
	sceIdStorageWriteLeaf(0x4, leaf);

	sceIdStorageReadLeaf(0x6,leaf);
	leaf[1]=0x68;
	sceIdStorageWriteLeaf(0x5, leaf);
	

	sceIdStorageWriteLeaf(0x6, idkey_6_82);

	pspSdkSetK1(k1);
	return 0;	
}

int pspTA082CanRun15Ipl()
{
	int k1 = pspSdkSetK1(0);
	
	sceIdStorageReadLeaf(0x5,leaf);


	memcpy(&leaf2, &idkey_5_82,512);
	// Change second byte to original (sony)
	leaf2[1]=0x6b;
	// if they're different, good (first 4 bytes only)
	if (memcmp(leaf,leaf2, 4))
	{		
		pspSdkSetK1(k1);
		return 1;
	}
	pspSdkSetK1(k1);
	return 0;
}

int pspTA082IsTA082()
{
	int k1 = pspSdkSetK1(0);

	sceIdStorageReadLeaf(0x100, leaf);

	if (leaf[0x3F] == 2)
	{
		sceIdStorageReadLeaf(0x50, leaf);

		if (leaf[0x21] == 2)
		{		
			pspSdkSetK1(k1);
			return 1;
		}
	}

	pspSdkSetK1(k1);
	return 0;
}

int isFinalDGed()
{
	int k1 = pspSdkSetK1(0);
	
	sceIdStorageReadLeaf(0x5,leaf);

	if (!memcmp(leaf,idkey_5_82,512))
	{
		pspSdkSetK1(k1);
		return 1; //True
	}
	pspSdkSetK1(k1);
	return 0; //False
}

int is4right()
{
	int k1 = pspSdkSetK1(0);

	sceIdStorageReadLeaf(0x4,leaf);

	if (!memcmp(leaf,idkey_4_82,512)) //memcmp returns 0 if they are the same
	{
		pspSdkSetK1(k1);
		return 1; //True
	}
	
	pspSdkSetK1(k1);
	return 0; //False
}

int is6right()
{
	int k1 = pspSdkSetK1(0);

	sceIdStorageReadLeaf(0x6,leaf);

	if (!memcmp(leaf,idkey_6_82,512)) //memcmp returns 0 if they are the same
	{
		pspSdkSetK1(k1);
		return 1; //True
	}
	
	pspSdkSetK1(k1);
	return 0; //False
}

int is5good()
{
	int k1 = pspSdkSetK1(0);

	sceIdStorageReadLeaf(0x5,leaf);
	

	if (!memcmp(leaf,idkey_5_82,512)) //memcmp returns 0 if they are the same
	{
		pspSdkSetK1(k1);
		return 1; //True
	}

	pspSdkSetK1(k1);
	return 0; //False
}

int is43right()
{
	int k1 = pspSdkSetK1(0);

	sceIdStorageReadLeaf(0x43,leaf);

	if (!memcmp(leaf,idkey_43,512)) //memcmp returns 0 if they are the same
	{
		pspSdkSetK1(k1);
		return 1; //True
	}
	
	pspSdkSetK1(k1);
	return 0; //False
}

int is47right()
{
	int k1 = pspSdkSetK1(0);

	
	memset(leaf,0,512);
	leaf[0] = 0x09;
	sceIdStorageReadLeaf(0x47,leaf2);
	

	//sceIdStorageLookup(0x43, 0, &key, 1);
	//if (key == 0x09)
	
	if (!memcmp(leaf,leaf2,512)) //memcmp returns 0 if they are the same
	{
		pspSdkSetK1(k1);
		return 1; //True
	}
	
	pspSdkSetK1(k1);
	return 0; //False
}

int is45right()
{
	int k1 = pspSdkSetK1(0);
	int key;

	sceIdStorageLookup(0x45,0,&key,3);
	if (key == 0x010000)
	{
		pspSdkSetK1(k1);
		return 1;
	}

	pspSdkSetK1(k1);
	return 0;
}



int is41right()
{
	int k1 = pspSdkSetK1(0);
	int key;

	sceIdStorageLookup(0x41, 0, &key, 4);
	if (key == 0x0000054C)
	// 13 == 0x4C0500000A0353006F006E0079
	{
		sceIdStorageLookup(0x41, 0x4C, &key, 4);
		if (key == 0x00500316)
		//21 == 0x160350005300500020005400790070006500200041
		{
			pspSdkSetK1(k1);
			return 1;
		}
	}	
	pspSdkSetK1(k1);
	return 0;
}

int is42backup6()
{
	int k1 = pspSdkSetK1(0);

	sceIdStorageReadLeaf(0x42,leaf);

	if (!memcmp(leaf,idkey_6_82,512)) //memcmp returns 0 if they are the same
	{
		pspSdkSetK1(k1);
		return 1; //True
	}
	
	pspSdkSetK1(k1);
	return 0; //False
}
int module_start(SceSize args, void *argp)
{
	return 0;
}

int module_stop(void)
{
	return 0;
}
